#define LEDPIN_A 4      //LED-A,数字端口D4
#define LEDPIN_B 5      //LED-B,数字端口D5
#define LEDPIN_C 6      //LED-C,数字端口D6

#define SENSOR_GP2Y1010_PIN 0    //Gp2y1010传感器,模拟端口A0
#define SENSOR_GP2Y1010_LEDPIN 2 //Gp2y1010传感器,数字端口D2

#define LIST_SIZE 8              //数据队列数量

double sensorGp2y1010ValList[LIST_SIZE];
int sensorGp2y1010NowIndex = 0;

void setup() {
  Serial.begin(9600);
  //开启LED灯端口输出模式
  pinMode(LEDPIN_A, OUTPUT);
  pinMode(LEDPIN_B, OUTPUT);
  pinMode(LEDPIN_C, OUTPUT);
  //开启Gp2y1010LED灯端口输出模式
  pinMode(SENSOR_GP2Y1010_LEDPIN, OUTPUT);

  for (int i = 0; i < LIST_SIZE; i++) {
    sensorGp2y1010ValList[i] = 0;
  }

}

void loop() {
  double nowNumGp2y1010 = 0;

  nowNumGp2y1010 = getGp2y1010Value();
  delay(400);
  openLed(nowNumGp2y1010);
  Serial.print("now: ");
  Serial.println(nowNumGp2y1010);
}

// 取Gp2y1010传感器数据
float getGp2y1010Value() {
  double nowNumGp2y1010 = 0;
  float dust_voltage = 0;     //
  int dust_analog;            //采集到的Gp2y1010传感器电压
  digitalWrite(SENSOR_GP2Y1010_LEDPIN, LOW);      //打开Gp2y1010LED灯
  delayMicroseconds(280);                         //开启LED后的280us的等待时间
  dust_analog = analogRead(SENSOR_GP2Y1010_PIN);  //读取模拟值
  delayMicroseconds(40);                          //40us等待时间
  digitalWrite(SENSOR_GP2Y1010_LEDPIN, HIGH);     //关闭Gp2y1010LED灯
  delayMicroseconds(9680);
  dust_voltage = dust_analog * 5.0 / 1023.0 ;     //将模拟值转换为电压值
  delay(200);   //GP2Y1014等待500毫秒，GP2Y1010等待200毫秒，
  
  sensorGp2y1010ValList[sensorGp2y1010NowIndex] = dust_voltage;
  sensorGp2y1010NowIndex ++;
  if (sensorGp2y1010NowIndex >= LIST_SIZE) {
    sensorGp2y1010NowIndex = 0;
  }

  for (int i = 0; i < LIST_SIZE; i++) {
    nowNumGp2y1010 += sensorGp2y1010ValList[i];
  }
  
  nowNumGp2y1010 /= LIST_SIZE;

  int concSHARP = 172 * nowNumGp2y1010 - 100;       //转换为ug/m3
  concSHARP = constrain(concSHARP, 0, 500);

  return concSHARP;
}

//打开灯
void openLed(double airQualityNumber)
{
  if (airQualityNumber <= 12) {
    //优（绿）
    digitalWrite(LEDPIN_A, HIGH);
    digitalWrite(LEDPIN_B, LOW);
    digitalWrite(LEDPIN_C, LOW);
  } else if (airQualityNumber > 12 && airQualityNumber <= 35.4) {
    //良（黄）
    digitalWrite(LEDPIN_A, LOW);
    digitalWrite(LEDPIN_B, HIGH);
    digitalWrite(LEDPIN_C, LOW);
  } else if (airQualityNumber > 35.4 && airQualityNumber <= 55.4) {
    //轻度污染（黄绿）
    digitalWrite(LEDPIN_A, HIGH);
    digitalWrite(LEDPIN_B, HIGH);
    digitalWrite(LEDPIN_C, LOW);
  } else if (airQualityNumber > 55.4 && airQualityNumber <= 150.4) {
    //中度污染（红）
    digitalWrite(LEDPIN_A, LOW);
    digitalWrite(LEDPIN_B, LOW);
    digitalWrite(LEDPIN_C, HIGH);
  } else if (airQualityNumber > 150.4 && airQualityNumber <= 250.4) {
    //重度污染（红绿）
    digitalWrite(LEDPIN_A, HIGH);
    digitalWrite(LEDPIN_B, LOW);
    digitalWrite(LEDPIN_C, HIGH);
  } else if (airQualityNumber > 250.4 && airQualityNumber <= 500.4 ) {
    //危险
    digitalWrite(LEDPIN_A, LOW);
    digitalWrite(LEDPIN_B, HIGH);
    digitalWrite(LEDPIN_C, HIGH);
  } else if (airQualityNumber > 500.4) {
    //爆表
    digitalWrite(LEDPIN_A, HIGH);
    digitalWrite(LEDPIN_B, HIGH);
    digitalWrite(LEDPIN_C, HIGH);
  }
}
